#include <stdlib.h>
#include <ps.h>
#include <vmtypes.h>
#include <bdaddr.h>
#include <byte_utils.h>

#include "sink_private_data.h"
#include "sink_dut.h"
#include "sink_malloc_debug.h"
#include "sink_configmanager.h"

#include <local_device.h>
#include "sink_audio_routing.h"
#include "sink_volume.h"
#include "sink_a2dp.h"
#include "sink_wired.h"
#include "vmal.h"
#include "sink_main_task.h"
#include "rtime.h"
#include "sink_audio_indication.h"
#include "sink_pio.h"
#include "sink_i2c.h"
#include "driver_tpa5828.h"
#include "sink_user.h"


#define BIT(x) (1 << (x))
#define TAS5828_DEVICE_CTRL_2 0x03

#define TSA5828_VOL_MAX 32


static const uint32_t TAS5828_VOL_TABLE[TSA5828_VOL_MAX + 1] =
{
    0x0000001b, // Volume = 0, -110db
    0x0000679f, // Volume = 1  -50db
    0x0000b845, // Volume = 2  -45db
    0x000147ae, // Volume = 3  -40db
    0x00019c86, // Volume = 4  -38db
    0x00020756, // Volume = 5  -36db
    0x00028dcf, // Volume = 6  -34db
    0x00033718, // Volume = 7  -32db
    0x00040c37, // Volume = 8  -30db
    0x00066a4a, // Volume = 9  -26db
    0x00081385, // Volume = 10 -24db
    0x00090fcc, // Volume = 11 -23db
    0x000a2adb, // Volume = 12 -22db
    0x000b6873, // Volume = 13 -21db
    0x000ccccd, // Volume = 14 -20db
    0x000e5ca1, // Volume = 15 -19db
    0x00101d3f, // Volume = 16 -18db
    0x0012149a, // Volume = 17 -17db
    0x00144961, // Volume = 18 -16db
    0x0016c311, // Volume = 19 -15db
    0x00198a13, // Volume = 20 -14db
    0x001ca7d7, // Volume = 21 -13db
    0x002026f3, // Volume = 22 -12db
    0x00241347, // Volume = 23 -11db
    0x00287a27, // Volume = 24 -10db
    0x002d6a86, // Volume = 25 -09db
    0x0032f52d, // Volume = 26 -08db
    0x00392cee, // Volume = 27 -07db
    0x004026e7, // Volume = 28 -06db
    0x0047facd, // Volume = 29 -05db
    0x0050c336, // Volume = 30 -04db
    0x00721483, // Volume = 31 -01db
    0x00800000, // Volume = 32 +00db
};

static uint8_t bsp_tas5828_sfr_write(uint8_t addr, uint8_t dat)
{
  return user_i2c_write_byte(addr, dat);
}

static uint8_t bsp_tas5828_sfr_read(uint8_t addr)
{
  uint8_t dat;
  user_i2c_read(addr, &dat, 1);
  return dat;
}

static void bsp_tas5828_write_buffer(uint8_t addr, uint8_t *buffer, uint8_t len)
{
  uint8_t sBuf[21];
  sBuf[0] = addr;
  memcpy(&sBuf[1], buffer, len);
  user_i2c_write_buffer(sBuf, len + 1);
}

static rtime_t SystemClockGetTimerTime(void)
{
  return (rtime_t)VmGetTimerTime();
}

static void tas5828_delay(uint16_t v_msec_16)
{
  uint32_t v_delay = SystemClockGetTimerTime() + (v_msec_16 * 1000) + 1;

  while (((int32_t)(SystemClockGetTimerTime() - v_delay)) < 0)
    ;
}

uint8_t TAS5828_SendInitCmd( uint8_t DevAddr , const tas5828_cmd_t *cmd, uint32_t cmd_size)
{
	uint32_t i;
	uint8_t ret = 0xff , error = 0;

init_start:

	user_i2c_change_device_addr(DevAddr);
	for (i = 0; i < cmd_size; i++)
	{
		if (cmd[i].cmd == CFG_META_DELAY)
		{
			tas5828_delay(cmd[i].parm);
		}
		else if (cmd[i].cmd == CFG_META_BURST)
		{
			user_i2c_write_buffer((unsigned char *)&cmd[i+1], cmd[i].parm);
            i +=  (cmd[i].parm / 2) + 1;
		}
		else
		{
			ret = user_i2c_write_buffer((unsigned char *)&cmd[i], 2);
			//ret = bsp_tas5828_sfr_write(cmd[i].cmd, cmd[i].parm);
			if (ret != 0)
			{
				break;
			}
		}
	}

	if (ret == 0)
	{
		printf("******** TAS5828 0x%x DriverInit Success**********\n", DevAddr);
		if(DevAddr == TAS_SPEAKER)
		{
			MessageCancelAll(&theSink.task, EventTas5828InitSuccess);
			MessageSend(&theSink.task, EventTas5828InitSuccess, NULL);
		}
	}
	else
	{
		if(++error < 3)
		{
			printf("******** TAS5828 0x%x DriverInit Error**********%d\n", DevAddr, ret);
		  	goto init_start;
		}

		if(DevAddr == TAS_SPEAKER)
		{
			MessageCancelAll(&theSink.task, EventTas5828Init);
			MessageSendLater(&theSink.task, EventTas5828Init, NULL, 2000);
		}
	}
	return ret;
}

/*uint8_t TAS5828_SendInitCmd( uint8_t DevAddr , const tas5828_cmd_t *cmd, uint32_t cmd_size)
{
	uint32_t i;
	uint8_t ret = 0xff , error = 0;

init_start:

	user_i2c_change_device_addr(DevAddr);
	for (i = 0; i < cmd_size; i++)
	{
		if (cmd[i].cmd == CFG_META_DELAY)
		{
			tas5828_delay(cmd[i].parm);
		}
		else
		{
			ret = bsp_tas5828_sfr_write(cmd[i].cmd, cmd[i].parm);
			if (ret != 0)
			{
				break;
			}
		}
	}

	if (ret == 0)
	{
		printf("******** TAS5828 0x%x DriverInit Success**********\n", DevAddr);
		if(DevAddr == TAS_SPEAKER)
		{
			MessageCancelAll(&theSink.task, EventTas5828InitSuccess);
			MessageSend(&theSink.task, EventTas5828InitSuccess, NULL);
		}
	}
	else
	{
		if(++error < 3)
		{
			printf("******** TAS5828 0x%x DriverInit Error**********%d\n", DevAddr, ret);
		  	goto init_start;
		}

		if(DevAddr == TAS_SPEAKER)
		{
			MessageCancelAll(&theSink.task, EventTas5828Init);
			MessageSendLater(&theSink.task, EventTas5828Init, NULL, 2000);
		}
	}
	return ret;
}*/

static void bsp_tas5828_play(void)
{
  uint8_t rdat;
  rdat = bsp_tas5828_sfr_read(TAS5828_DEVICE_CTRL_2);
  rdat |= BIT(0) | BIT(1);
  bsp_tas5828_sfr_write(TAS5828_DEVICE_CTRL_2, rdat);
}

static void bsp_tas5828_sleep(void)
{
  uint8_t rdat;
  rdat = bsp_tas5828_sfr_read(TAS5828_DEVICE_CTRL_2);
  rdat &= ~(BIT(0) | BIT(1));
  bsp_tas5828_sfr_write(TAS5828_DEVICE_CTRL_2, rdat);
}

static void bsp_tas5828_Hiz(void)
{
  uint8_t rdat;
  rdat = bsp_tas5828_sfr_read(TAS5828_DEVICE_CTRL_2);
  rdat &= ~(BIT(0) | BIT(1));
  rdat |= BIT(1);
  bsp_tas5828_sfr_write(TAS5828_DEVICE_CTRL_2, rdat);
}

static void bsp_tas5828_set_vol(uint8_t vol)
{
  uint32_t vol_value;

  if (vol > TSA5828_VOL_MAX)
    vol = TSA5828_VOL_MAX;
  vol_value = TAS5828_VOL_TABLE[vol];

  bsp_tas5828_sfr_write(0x00, 0x00); // page 0
  bsp_tas5828_sfr_write(0x7F, 0x8C); // book 8c
  bsp_tas5828_sfr_write(0x00, 0x2A); // page 2A

  uint8_t buf[4];
  buf[0] = (vol_value >> 24) & 0xff;
  buf[1] = (vol_value >> 16) & 0xff;
  buf[2] = (vol_value >> 8) & 0xff;
  buf[3] = (vol_value >> 0) & 0xff;
  bsp_tas5828_write_buffer(0x24, buf, 4);
  bsp_tas5828_write_buffer(0x28, buf, 4);

  bsp_tas5828_sfr_write(0x00, 0x00); // page 0
  bsp_tas5828_sfr_write(0x7F, 0x00); // book 0
}

static void bsp_tas5828_mute(void)
{
  uint8_t rdat;

  bsp_tas5828_sfr_write(0x00, 0x00);
  bsp_tas5828_sfr_write(0x7F, 0x00);
  bsp_tas5828_sfr_write(0x00, 0x00);
  rdat = bsp_tas5828_sfr_read(TAS5828_DEVICE_CTRL_2);
  rdat |= BIT(3);
  bsp_tas5828_sfr_write(0x00, 0x00);
  bsp_tas5828_sfr_write(0x7F, 0x00);
  bsp_tas5828_sfr_write(0x00, 0x00);
  bsp_tas5828_sfr_write(TAS5828_DEVICE_CTRL_2, rdat);
}

static void bsp_tas5828_unmute(void)
{
  uint8_t rdat;
  bsp_tas5828_sfr_write(0x00, 0x00);
  bsp_tas5828_sfr_write(0x7F, 0x00);
  bsp_tas5828_sfr_write(0x00, 0x00);  
  rdat = bsp_tas5828_sfr_read(TAS5828_DEVICE_CTRL_2);
  rdat &= ~BIT(3);
  bsp_tas5828_sfr_write(0x00, 0x00);
  bsp_tas5828_sfr_write(0x7F, 0x00);
  bsp_tas5828_sfr_write(0x00, 0x00);  
  bsp_tas5828_sfr_write(TAS5828_DEVICE_CTRL_2, rdat);
}

static void bsp_tas5828_clear_fault(void)
{
  bsp_tas5828_sfr_write(0x78, 0x80);
}

void TAS5828_SendVolume(uint8_t DevAddr, uint8_t vol)
{
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_set_vol(vol);
}

uint8_t bass_buf[16][20] = 
{
	0x07,0xf8,0xca,0x9c,0xf0,0x15,0x2c,0x06,0x07,0xf2,0x1f,0xb6,0x0f,0xea,0xd3,0xfa,0xf8,0x15,0x15,0xae,//-10
	0x07,0xfa,0xf4,0x51,0xf0,0x10,0xda,0x71,0x07,0xf4,0x47,0x9c,0x0f,0xef,0x25,0x8f,0xf8,0x10,0xc4,0x13,//-8
	0x07,0xfb,0xdd,0x39,0xf0,0x0f,0x09,0x65,0x07,0xf5,0x2f,0xc2,0x0f,0xf0,0xf6,0x9b,0xf8,0x0e,0xf3,0x05,//-7
	0x07,0xfc,0xac,0xfa,0xf0,0x0d,0x6a,0x93,0x07,0xf5,0xfe,0xd5,0x0f,0xf2,0x95,0x6d,0xf8,0x0d,0x54,0x30,//-6
	0x07,0xfd,0x66,0x47,0xf0,0x0b,0xf8,0x96,0x07,0xf6,0xb7,0x87,0x0f,0xf4,0x07,0x6a,0xf8,0x0b,0xe2,0x31,//-5
	0x07,0xfe,0x9e,0xea,0xf0,0x09,0x88,0x5a,0x07,0xf7,0xef,0x24,0x0f,0xf6,0x77,0xa6,0xf8,0x09,0x71,0xf1,//-3
	0x07,0xff,0x22,0x55,0xf0,0x08,0x81,0xf2,0x07,0xf8,0x72,0x22,0x0f,0xf7,0x7e,0x0e,0xf8,0x08,0x6b,0x89,//-2
	0x07,0xff,0x97,0x84,0xf0,0x07,0x97,0xf8,0x07,0xf8,0xe6,0xef,0x0f,0xf8,0x68,0x08,0xf8,0x07,0x81,0x8d,//-1
	0x08,0x00,0x00,0x00,0xf0,0x06,0xc7,0x59,0x07,0xf9,0x4f,0x13,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//0
	0x08,0x00,0x68,0x81,0xf0,0x06,0xc7,0x59,0x07,0xf8,0xe6,0x92,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//1
	0x08,0x00,0xdd,0xc2,0xf0,0x06,0xc7,0x59,0x07,0xf8,0x71,0x50,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//2
	0x08,0x01,0x61,0x53,0xf0,0x06,0xc7,0x59,0x07,0xf7,0xed,0xc0,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//3
	0x08,0x02,0x9a,0x91,0xf0,0x06,0xc7,0x59,0x07,0xf6,0xb4,0x82,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//5
	0x08,0x03,0x54,0x68,0xf0,0x06,0xc7,0x59,0x07,0xf5,0xfa,0xab,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//6
	0x08,0x04,0x24,0xeb,0xf0,0x06,0xc7,0x59,0x07,0xf5,0x2a,0x28,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//7
	0x08,0x05,0x0e,0xe0,0xf0,0x06,0xc7,0x59,0x07,0xf4,0x40,0x33,0x0f,0xf9,0x38,0xa7,0xf8,0x06,0xb0,0xed,//8
};

void TAS5828_bass_SendVolume(uint8_t DevAddr, uint8_t vol)
{
	user_i2c_change_device_addr(DevAddr);

	bsp_tas5828_sfr_write(0x00, 0x00); 
	bsp_tas5828_sfr_write(0x7F, 0xAA); 
	
	bsp_tas5828_sfr_write(0x00, 0x24);
	bsp_tas5828_write_buffer(0x40, bass_buf[vol], 20);
}

uint8_t tre_buf_1[16][4] = 
{
	0x03,0xcd,0x30,0x07,//-13
	0x04,0x6a,0x65,0x44,//-11
	0x04,0xbc,0xab,0xdc,//-10
	0x05,0x10,0xcb,0x3c,//-9
	0x05,0xbc,0x80,0xc4,//-7
	0x06,0x69,0x1c,0x59,//-5
	0x07,0x12,0x1c,0xc5,//-3
	0x07,0x63,0xf1,0x37,//-2
	0x07,0xb3,0x5e,0xd2,//-1
	0x08,0x00,0x00,0x00,//0
	0x08,0x4f,0x9b,0xb9,//1
	0x08,0xa8,0xee,0x25,//2
	0x09,0x0d,0x26,0xb2,//3
	0x09,0x7d,0x99,0xd1,//4
	0x09,0xfb,0xc5,0x80,//5
	0x0a,0x89,0x56,0x57,//6	
};

uint8_t tre_buf_2[16][16] = 
{
	0xfb,0xe3,0xc2,0x2c,0x01,0x61,0x22,0x41,0x04,0x1c,0x3d,0xd4,0x02,0xd1,0xad,0xb9,//-13
	0xfb,0x39,0xc7,0xbc,0x01,0x9a,0x2d,0xc3,0x04,0xc6,0x38,0x44,0x01,0xfb,0x6c,0xf9,//-11
	0xfa,0xe0,0xd2,0x2f,0x01,0xb8,0x08,0x99,0x05,0x1f,0x2d,0xd1,0x01,0x8b,0x4b,0x8a,//-10
	0xfa,0x85,0xdd,0x73,0x01,0xd6,0x8e,0xfd,0x05,0x7a,0x22,0x8d,0x01,0x18,0xa5,0xc7,//-9
	0xf9,0xcc,0x35,0x2a,0x02,0x14,0xdd,0x8d,0x06,0x33,0xca,0xd6,0x00,0x2e,0xa1,0xaf,//-7
	0xf9,0x11,0x94,0x24,0x02,0x53,0x7f,0x97,0x06,0xee,0x6b,0xdc,0xff,0x43,0x64,0x10,//-5
	0xf8,0x5a,0xd9,0x44,0x02,0x90,0xd2,0xa7,0x07,0xa5,0x26,0xbc,0xfe,0x5d,0x10,0x94,//-3
	0xf8,0x02,0x5f,0x23,0x02,0xae,0x84,0x11,0x07,0xfd,0xa0,0xdd,0xfd,0xed,0x8a,0xb8,//-2
	0xf7,0xac,0x7d,0xcb,0x02,0xcb,0x56,0x62,0x08,0x53,0x82,0x35,0xfd,0x81,0x4a,0xcc,//-1
	0xf7,0x59,0xa3,0x12,0x02,0xe7,0x24,0xbb,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//0
	0xf7,0x59,0xa3,0x12,0x02,0x97,0x89,0x02,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//1
	0xf7,0x59,0xa3,0x12,0x02,0x3e,0x36,0x96,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//2
	0xf7,0x59,0xa3,0x12,0x01,0xd9,0xfe,0x09,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//3
	0xf7,0x59,0xa3,0x12,0x01,0x69,0x8a,0xea,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//4
	0xf7,0x59,0xa3,0x12,0x00,0xeb,0x5f,0x3b,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//5
	0xf7,0x59,0xa3,0x12,0x00,0x5d,0xce,0x64,0x08,0xa6,0x5c,0xee,0xfd,0x18,0xdb,0x45,//6
};

void TAS5828_tre_SendVolume(uint8_t DevAddr, uint8_t vol)
{
	user_i2c_change_device_addr(DevAddr);

	bsp_tas5828_sfr_write(0x00, 0x00); 
	bsp_tas5828_sfr_write(0x7F, 0xAA); 
	bsp_tas5828_sfr_write(0x00, 0x25); 

	bsp_tas5828_write_buffer(0x7c, tre_buf_1[vol], 4);

	bsp_tas5828_sfr_write(0x00, 0x00); 
	bsp_tas5828_sfr_write(0x7F, 0xAA); 
	bsp_tas5828_sfr_write(0x00, 0x26); 	

	bsp_tas5828_write_buffer(0x08, tre_buf_2[vol], 16);
}

uint8_t sum_buf[16][4] = 
{
	0x00,0x00,0x00,0x1b,//-110
	0x00,0x03,0x37,0x18,//-14
	0x00,0x05,0xb7,0xb1,//-13
	0x00,0x0a,0x2a,0xdb,//-12
	0x00,0x0c,0xcc,0xcd,//-11
	0x00,0x10,0x1d,0x3f,//-10
	0x00,0x14,0x49,0x61,//-9
	0x00,0x19,0x8a,0x13,//-8
	0x00,0x20,0x26,0xf3,//-7
	0x00,0x28,0x7a,0x27,//-6
	0x00,0x32,0xf5,0x2d,//-5
	0x00,0x39,0x2c,0xee,//-4
	0x00,0x40,0x26,0xe7,//-3
	0x00,0x50,0xc3,0x36,//-2
	0x00,0x65,0xac,0x8c,//-1
	0x00,0x80,0x00,0x00,//0
};

void TAS5828_sum_SendVolume(uint8_t DevAddr, uint8_t vol)
{
	user_i2c_change_device_addr(DevAddr);
	
	bsp_tas5828_sfr_write(0x00, 0x00); // page 0
	bsp_tas5828_sfr_write(0x7F, 0x8C); // book 8c
	bsp_tas5828_sfr_write(0x00, 0x2A); // page 2A
	bsp_tas5828_write_buffer(0x24, sum_buf[vol], 4);
	
	bsp_tas5828_sfr_write(0x00, 0x00); // page 0
	bsp_tas5828_sfr_write(0x7F, 0x8C); // book 8c
	bsp_tas5828_sfr_write(0x00, 0x2A); // page 2A	
	bsp_tas5828_write_buffer(0x28, sum_buf[vol], 4);
}

void TAS5828_bass_open(uint8_t DevAddr)
{
	uint8_t data_1[20] = {0x08,0x00,0x00,0x00,0xf0,0xbc,0xdb,0x13,0x07,0x4b,0x87,0x16,0x0f,0x43,0x24,0xed,0xf8,0xb4,0x78,0xea};
	uint8_t data_2[16] = {0x08,0x05,0xad,0xb6,0xf0,0x06,0x49,0xe5,0x07,0xf4,0x13,0x62,0x0f,0xf9,0xb6,0x1b};
	uint8_t data_3[4] = {0xF8,0x06,0x3E,0xe8};	
	uint8_t data_4[20] = {0x07,0xfa,0xa6,0x1b,0xf0,0x0a,0xb3,0xc9,0x07,0xfa,0xa6,0x1b,0x0f,0xf5,0x4a,0x6c,0xf8,0x0a,0xb1,0xff};

	user_i2c_change_device_addr(DevAddr);
	
	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2A); // 
	bsp_tas5828_write_buffer(0x5C, data_1, 20);

	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2A); // 
	bsp_tas5828_write_buffer(0x70, data_2, 16);
	
	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2B); // 	
	bsp_tas5828_write_buffer(0x08, data_3, 4);

	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2E); // 	
	bsp_tas5828_write_buffer(0x40, data_4, 20);	
}

void TAS5828_DRC_open(uint8_t DevAddr)
{
	user_i2c_change_device_addr(DevAddr);
	
	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0x00); // 
	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x66, 0x04); // 
}


void TAS5828_BASS_DRC_open(uint8_t DevAddr)
{
	uint8_t data_1[40] = {0x00,0x0d,0xa6,0x86,0x00,0x0d,0xa6,0x86,0x00,0x06,0xd3,0x72,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x81,0x47,0xae,0xf9,0xda,0xbc,0x21,0xfd,0x2d,0x26,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	uint8_t data_2[20] = {0x00,0x00,0xc8,0xbc,0x00,0x00,0xc8,0xbc,0x00,0x00,0xc8,0xbc,0x7f,0x1c,0x87,0x2b,0x81,0xc3,0xce,0xba};
	uint8_t data_3[20] = {0x00,0x00,0xc8,0xbc,0x00,0x00,0xc8,0xbc,0x00,0x00,0xc8,0xbc,0x7f,0x1c,0x87,0x2b,0x81,0xc3,0xce,0xba};	
	uint8_t data_4[4] = {0x00,0x80,0x00,0x00};

	user_i2c_change_device_addr(DevAddr);
	
	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0x8C); // 
	bsp_tas5828_sfr_write(0x00, 0x2B); // 
	bsp_tas5828_write_buffer(0x34, data_1, 40);

	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2A); // 
	bsp_tas5828_write_buffer(0x34, data_2, 20);
	
	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2A); // 	
	bsp_tas5828_write_buffer(0x48, data_3, 20);

	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x7F, 0x8C); // 
	bsp_tas5828_sfr_write(0x00, 0x2E); // 	
	bsp_tas5828_write_buffer(0x08, data_4, 4);	
}


void TAS5828_TRE_DRC_open(uint8_t DevAddr)
{
	uint8_t data_1[40] = {0x01,0x53,0x8f,0xcc,0x01,0x53,0x8f,0xcc,0x00,0x03,0x69,0xc5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x81,0x47,0xae,0xf9,0xda,0xbc,0x21,0xfd,0x2d,0x26,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	uint8_t data_2[20] = {0x7f,0x1d,0x4f,0xe7,0x80,0xe2,0xb0,0x19,0x7f,0x1d,0x4f,0xe7,0x7f,0x1c,0x87,0x2b,0x81,0xc3,0xce,0xba};
	uint8_t data_3[20] = {0x7f,0x1d,0x4f,0xe7,0x80,0xe2,0xb0,0x19,0x7f,0x1d,0x4f,0xe7,0x7f,0x1c,0x87,0x2b,0x81,0xc3,0xce,0xba};	
	uint8_t data_4[4] = {0x00,0x80,0x00,0x00};

	user_i2c_change_device_addr(DevAddr);
	
	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0x8C); // 
	bsp_tas5828_sfr_write(0x00, 0x2D); // 
	bsp_tas5828_write_buffer(0x58, data_1, 40);

	bsp_tas5828_sfr_write(0x00, 0x00); //
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2B); // 
	bsp_tas5828_write_buffer(0x0C, data_2, 20);
	
	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x7F, 0xAA); // 
	bsp_tas5828_sfr_write(0x00, 0x2B); // 	
	bsp_tas5828_write_buffer(0x20, data_3, 20);

	bsp_tas5828_sfr_write(0x00, 0x00); // 
	bsp_tas5828_sfr_write(0x7F, 0x8C); // 
	bsp_tas5828_sfr_write(0x00, 0x2E); // 	
	bsp_tas5828_write_buffer(0x10, data_4, 4);	
}

void TAS5828_Play(uint8_t DevAddr)
{
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_play();
}

void TAS5828_Sleep(uint8_t DevAddr)
{
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_sleep();
}
void TAS5828_Hiz(uint8_t DevAddr)
{
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_Hiz();
}
//static bool spkeaer_mute_flag = 0;
//static bool sub_mute_flag = 0;
void TAS5828_Mute(uint8_t DevAddr)
{
	if(!SinkUser.DspIsOpenSuccess)
	{
		return;
	}
	
	if(SinkUser.DspIsMuteFlag == TRUE && DevAddr == TAS_SPEAKER)
	{
		return ;
	}

	SinkUser.DspIsMuteFlag = TRUE;
  
  /*if(spkeaer_mute_flag && DevAddr == TAS_SPEAKER)
  {
    return ;
  }
  if(sub_mute_flag && DevAddr == TAS_SUB)
  {
    return ;
  }*/
  printf("---TAS5828 Mute 0x%X\r\n" , DevAddr);
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_mute();
  //bsp_tas5828_sleep();
  /*if(DevAddr == TAS_SPEAKER)
  {
    spkeaer_mute_flag = 1;
  }
  if(DevAddr == TAS_SUB)
  {
    sub_mute_flag = 1;
  }*/
  
}

void TAS5828_Vol_Min_Mute(uint8_t DevAddr)
{

  printf("---TAS5828 vol min Mute 0x%X\r\n" , DevAddr);
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_mute();
}

void TAS5828_Unmute(uint8_t DevAddr)
{
	if(!SinkUser.DspIsOpenSuccess)
	{
	    return;
	}
	
	if(SinkUser.DspIsMuteFlag == FALSE && DevAddr == TAS_SPEAKER)
	{
		return ;
	}

	SinkUser.DspIsMuteFlag = FALSE;
	
  /*if(!spkeaer_mute_flag && DevAddr == TAS_SPEAKER)
  {
    return ;
  }
  if(!sub_mute_flag && DevAddr == TAS_SUB)
  {
    return ;
  }*/
  if(SinkUser.DspIsVolMinFlag == FALSE)
  {
	  printf("---TAS5828 Unmute 0x%X\r\n" , DevAddr);
	  user_i2c_change_device_addr(DevAddr);
	  bsp_tas5828_unmute();
  }
  //bsp_tas5828_play();
  /*if(DevAddr == TAS_SPEAKER)
  {
    spkeaer_mute_flag = 0;
  }
  if(DevAddr == TAS_SUB)
  {
    sub_mute_flag = 0;
  }*/
}

void TAS5828_ClearFault(uint8_t DevAddr)
{
  user_i2c_change_device_addr(DevAddr);
  bsp_tas5828_clear_fault();
}


void TAS5828_DriverDeInit(void)
{
  //tas5828_delay(1000);
}

void TAS5828_DriverInit(void)
{
  if (tas5828_speaker_init(TAS_SPEAKER)== 0)
  {
    //TAS5805_ClearFault(TPA5808_ADDR1);
    //TAS5805_Play(TPA5808_ADDR1);
    //TAS5805_SendVolume(TPA5808_ADDR1, TSA5805_VOL_MAX);
    //TAS5805_Mute(TPA5808_ADDR1);
    //printf("61h %02x\r\n" , bsp_tas5805_sfr_read(0x61));
  }
  //if (tas5828_sub_init(TAS_SUB)== 0)
  //{
    //TAS5805_ClearFault(TPA5808_ADDR2);
    //TAS5805_Play(TPA5808_ADDR2);
    //TAS5805_SendVolume(TPA5808_ADDR2, TSA5805_VOL_MAX);
    //TAS5805_Mute(TPA5808_ADDR2);
    //printf("61h %02x\r\n" , bsp_tas5805_sfr_read(0x61));
  //}
  //spkeaer_mute_flag = 0;
}
